"configuration objects global API"
import autopath
from py.test import raises

from tiramisu.config import Config
from tiramisu.option import IntOption, FloatOption, StrOption, ChoiceOption, \
    BoolOption, OptionDescription


def make_description():
    gcoption = ChoiceOption('name', 'GC name', ('ref', 'framework'), 'ref')
    gcdummy = BoolOption('dummy', 'dummy', default=False)
    objspaceoption = ChoiceOption('objspace', 'Object space',
                                  ('std', 'thunk'), 'std')
    booloption = BoolOption('bool', 'Test boolean option', default=True)
    intoption = IntOption('int', 'Test int option', default=0)
    floatoption = FloatOption('float', 'Test float option', default=2.3)
    stroption = StrOption('str', 'Test string option', default="abc")
    boolop = BoolOption('boolop', 'Test boolean option op', default=True)
    wantref_option = BoolOption('wantref', 'Tests', default=False)
    wantframework_option = BoolOption('wantframework', 'Test', default=False)
    gcgroup = OptionDescription('gc', '', [gcoption, gcdummy, floatoption])
    descr = OptionDescription('tiramisu', '', [gcgroup, booloption, objspaceoption,
                                               wantref_option, stroption,
                                               wantframework_option,
                                               intoption, boolop])
    return descr


def test_compare_configs():
    "config object comparison"
    descr = make_description()
    conf1 = Config(descr)
    conf2 = Config(descr)
    conf2.wantref = True
    assert conf1 != conf2
    assert hash(conf1) != hash(conf2)
    #assert conf1.getkey() != conf2.getkey()
    conf1.wantref = True
    assert conf1 == conf2
    assert hash(conf1) == hash(conf2)
    #assert conf1.getkey() == conf2.getkey()
    conf2.gc.dummy = True
    assert conf1 != conf2
    assert hash(conf1) != hash(conf2)
    #assert conf1.getkey() != conf2.getkey()
    conf1.gc.dummy = True
    assert conf1 == conf2
    assert hash(conf1) == hash(conf2)
    assert not conf1 == 'conf2'
    assert conf1 != 'conf2'
# ____________________________________________________________


def test_iter_config():
    "iteration on config object"
    s = StrOption("string", "", default="string")
    s2 = StrOption("string2", "", default="string2")
    descr = OptionDescription("options", "", [s, s2])
    config = Config(descr)
    assert [(name, value) for name, value in config] == \
        [('string', 'string'), ('string2', 'string2')]


def test_iter_subconfig():
    "iteration on config sub object"
    descr = make_description()
    conf = Config(descr)
    for (name, value), (gname, gvalue) in \
            zip(conf.gc, [("name", "ref"), ("dummy", False)]):
        assert name == gname
        assert value == gvalue


def test_str():
    descr = make_description()
    c = Config(descr)
    c  # does not crash


def test_make_dict():
    "serialization of the whole config to a dict"
    descr = OptionDescription("opt", "", [
        OptionDescription("s1", "", [
            BoolOption("a", "", default=False)]),
        IntOption("int", "", default=42)])
    config = Config(descr)
    d = config.make_dict()
    assert d == {"s1.a": False, "int": 42}
    config.int = 43
    config.s1.a = True
    d = config.make_dict()
    assert d == {"s1.a": True, "int": 43}
    d2 = config.make_dict(flatten=True)
    assert d2 == {'a': True, 'int': 43}
    raises(ValueError, 'd2 = config.make_dict(withvalue="3")')


def test_find_in_config():
    "finds option in config"
    descr = make_description()
    conf = Config(descr)
    assert conf.find(byname='dummy') == [conf.unwrap_from_path('gc.dummy')]
    assert conf.find_first(byname='dummy') == conf.unwrap_from_path('gc.dummy')
    assert conf.find(bytype=ChoiceOption) == [conf.unwrap_from_path('gc.name'), conf.unwrap_from_path('objspace')]
    assert conf.find_first(bytype=ChoiceOption) == conf.unwrap_from_path('gc.name')
    assert conf.find(byvalue='ref') == [conf.unwrap_from_path('gc.name')]
    assert conf.find_first(byvalue='ref') == conf.unwrap_from_path('gc.name')
    # combinaison of filters
    assert conf.find(bytype=BoolOption, byname='dummy') == [conf.unwrap_from_path('gc.dummy')]
    assert conf.find_first(bytype=BoolOption, byname='dummy') == conf.unwrap_from_path('gc.dummy')
    assert conf.find(byvalue=False, byname='dummy') == [conf.unwrap_from_path('gc.dummy')]
    assert conf.find_first(byvalue=False, byname='dummy') == conf.unwrap_from_path('gc.dummy')
    ## byattrs
    #assert conf.find_first(byattrs= dict(default=2.3)) == conf.unwrap_from_path('gc.float')
    #assert conf.find_first(byvalue=False, byname='dummy', byattrs=dict(default=False)) == conf.unwrap_from_path('gc.dummy')


def test_find_multi():
    b = BoolOption('bool', '', multi=True)
    o = OptionDescription('od', '', [b])
    conf = Config(o)
    raises(AttributeError, "conf.find(byvalue=True)")
    raises(AttributeError, "conf.find_first(byvalue=True)")
    conf.bool.append(False)
    raises(AttributeError, "conf.find(byvalue=True)")
    raises(AttributeError, "conf.find_first(byvalue=True)")
    conf.bool.append(False)
    raises(AttributeError, "conf.find(byvalue=True)")
    raises(AttributeError, "conf.find_first(byvalue=True)")
    conf.bool.append(True)
    assert conf.find(byvalue=True) == [b]
    assert conf.find_first(byvalue=True) == b


def test_does_not_find_in_config():
    descr = make_description()
    conf = Config(descr)
    raises(AttributeError, "conf.find(byname='IDontExist')")
